"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClassToPlainArrayTest = void 0;
const mocha_1 = require("@testdeck/mocha");
const chai_1 = require("chai");
const common_errors_1 = require("common-errors");
const src_1 = require("../src");
const CustomStorageClass_1 = require("./classes/CustomStorageClass");
const PassClassTransformOption_1 = require("./classes/PassClassTransformOption");
const Product_1 = require("./classes/Product");
const Ship_1 = require("./classes/Ship");
const SkipIndex_1 = require("./classes/SkipIndex");
const factories_1 = require("./factories");
class KeyNotInObject {
    constructor() {
        this.deleteThis = true;
    }
}
__decorate([
    src_1.ArrayMember(0),
    __metadata("design:type", Boolean)
], KeyNotInObject.prototype, "deleteThis", void 0);
class UnknownClass {
}
class ChildType {
}
class ParentType {
    constructor() {
        this.child = null;
    }
}
__decorate([
    src_1.ArrayMember(0),
    __metadata("design:type", Object)
], ParentType.prototype, "child", void 0);
let ClassToPlainArrayTest = class ClassToPlainArrayTest {
    normal() {
        const expected = factories_1.factory.make(Product_1.Product).one();
        const result = src_1.classToPlainArray(expected);
        Product_1.productArrayValidate(expected, result);
    }
    array() {
        const testData = factories_1.factory.make(Product_1.Product).many(2);
        const results = src_1.classToPlainArray(testData);
        chai_1.expect(results).property('constructor', Array);
        chai_1.expect(results).length(testData.length);
        for (const [i, expected] of testData.entries()) {
            chai_1.expect(results).property(i.toString());
            Product_1.productArrayValidate(expected, results[i]);
        }
    }
    emptyArray() {
        const result = src_1.classToPlainArray([]);
        chai_1.expect(result.constructor).equals(Array);
        chai_1.expect(result).length(0);
    }
    unknownClass() {
        chai_1.expect(() => src_1.classToPlainArray(new UnknownClass())).throw(src_1.UnknownClassError);
    }
    keyNotInObject() {
        const obj = new KeyNotInObject();
        delete obj.deleteThis;
        const result = src_1.classToPlainArray(obj);
        chai_1.expect(result.constructor).equals(Array);
        chai_1.expect(result).length(1);
        chai_1.expect(result).property('0').is.undefined;
    }
    missingChildType() {
        const result = src_1.classToPlainArray(new ParentType());
        chai_1.expect(result.constructor).equals(Array);
        chai_1.expect(result).length(1);
        chai_1.expect(result).property('0').is.null;
    }
    undefinedArray() {
        const expected = factories_1.factory.make(Product_1.Product).one();
        const result = src_1.classToPlainArray(Object.assign(expected, { sizes: null }));
        chai_1.expect(result).property('4').is.null;
    }
    arrayIsNotArray() {
        const expected = factories_1.factory.make(Product_1.Product).one();
        chai_1.expect(() => src_1.classToPlainArray(Object.assign(expected, { sizes: 1 }))).throw(common_errors_1.TypeError);
    }
    skipIndex() {
        const expected = factories_1.factory.make(SkipIndex_1.SkipIndex).one();
        SkipIndex_1.skipIndexArrayValidate(expected, src_1.classToPlainArray(expected));
    }
    classTransformOption() {
        const expected = factories_1.factory.make(PassClassTransformOption_1.PassClassTransformOption).one();
        const result = src_1.classToPlainArray(expected, { strategy: 'excludeAll' });
        PassClassTransformOption_1.passClassTransformOptionArrayValidate(expected, result);
    }
    customStorage() {
        const expected = factories_1.factory.make(CustomStorageClass_1.CustomStorageClass).one();
        CustomStorageClass_1.customStorageArrayValidate(expected, src_1.classToPlainArray(expected, { arrayMemberStorage: CustomStorageClass_1.arrayMemberStorage }));
    }
    customStorageFail() {
        const expected = factories_1.factory.make(CustomStorageClass_1.CustomStorageClass).one();
        chai_1.expect(() => src_1.classToPlainArray(expected)).throw(src_1.UnknownClassError);
    }
    customStorageDecorator() {
        const expected = factories_1.factory.make(Ship_1.Ship).one();
        Ship_1.shipArrayValidate(expected, src_1.classToPlainArray(expected, { arrayMemberStorage: Ship_1.shipStorage }));
    }
    customStorageDecoratorFail() {
        const expected = factories_1.factory.make(Ship_1.Ship).one();
        chai_1.expect(() => src_1.classToPlainArray(expected)).throw(src_1.UnknownClassError);
    }
    customStorageDecoratorWithDefault() {
        const expected = factories_1.factory.make(Ship_1.ShipWithDefault).one();
        Ship_1.shipWithDefaultArrayValidate(expected, src_1.classToPlainArray(expected, { arrayMemberStorage: Ship_1.shipStorage }));
        Ship_1.shipWithDefaultArrayValidate(expected, src_1.classToPlainArray(expected));
    }
    customStorageDecoratorPartial() {
        const expected = factories_1.factory.make(Ship_1.ShipWithPartialProperty).one();
        Ship_1.shipWithPartialPropertyArrayValidate(Object.assign(new Ship_1.ShipWithPartialProperty(), { ...expected, name: null }), src_1.classToPlainArray(expected, { arrayMemberStorage: Ship_1.shipStorage }));
        Ship_1.shipWithPartialPropertyArrayValidate(Object.assign(new Ship_1.ShipWithPartialProperty(), { ...expected, id: null }), src_1.classToPlainArray(expected, { arrayMemberStorage: Ship_1.shipStoragePartial }));
    }
    customStorageDecoratorPartialFail() {
        const expected = factories_1.factory.make(Ship_1.ShipWithPartialProperty).one();
        chai_1.expect(() => src_1.classToPlainArray(expected)).throw(src_1.UnknownClassError);
    }
};
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "normal", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "array", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "emptyArray", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "unknownClass", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "keyNotInObject", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "missingChildType", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "undefinedArray", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "arrayIsNotArray", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "skipIndex", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "classTransformOption", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "customStorage", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "customStorageFail", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "customStorageDecorator", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "customStorageDecoratorFail", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "customStorageDecoratorWithDefault", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "customStorageDecoratorPartial", null);
__decorate([
    mocha_1.test(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], ClassToPlainArrayTest.prototype, "customStorageDecoratorPartialFail", null);
ClassToPlainArrayTest = __decorate([
    mocha_1.suite()
], ClassToPlainArrayTest);
exports.ClassToPlainArrayTest = ClassToPlainArrayTest;
//# sourceMappingURL=classToPlainArray.test.js.map